// Kevin Pang
// Neural Networks
// Fall 2003
//
// main.cpp
// Self-organizing Map Demo driver file

#include <GL/glut.h>
#include <vector>
#include <time.h>
#include <iostream>
#include "som.h"

using namespace std;

void display(void);
void reshape(int width, int height);
void keyboard(unsigned char key, int x, int y);
void init();

void idle(void);

// The map and some variables
Som* som;
int size = 100;
int numInputs = 48;
int count = 0;

// The sample inputs
vector<vector<double> > input;

bool training;

main(int argc, char **argv)
{
  cout << "Self-organizing Map Demo Program" << endl;
  cout << "Written by Kevin Pang" << endl;
  cout << "http://www.cs.hmc.edu/~kpang/nn/som.html" << endl << endl;
  cout << "Commands:" << endl;
  cout << "Press the 't' key to toggle training on/off" << endl;
  cout << "Press the 'r' key to reset the map" << endl << endl;

  training = false;

  srand(time(NULL));

  // Create the map
  som = new Som(size, numInputs);

  // Set the inputs
  input.resize(numInputs);
  for (int i = 0; i < numInputs; ++i)
	  input[i].resize(3);

  // Brighest
  input[0][0] = 255;
  input[0][1] = 0;
  input[0][2] = 0;

  input[1][0] = 0;
  input[1][1] = 255;
  input[1][2] = 0;

  input[2][0] = 0;
  input[2][1] = 0;
  input[2][2] = 255;

  input[3][0] = 255;
  input[3][1] = 255;
  input[3][2] = 0;

  input[4][0] = 255;
  input[4][1] = 0;
  input[4][2] = 255;

  input[5][0] = 0;
  input[5][1] = 255;
  input[5][2] = 255;

  input[6][0] = 255;
  input[6][1] = 128;
  input[6][2] = 0;

  input[7][0] = 255;
  input[7][1] = 0;
  input[7][2] = 128;

  input[8][0] = 128;
  input[8][1] = 255;
  input[8][2] = 0;

  input[9][0] = 0;
  input[9][1] = 255;
  input[9][2] = 128;

  input[10][0] = 128;
  input[10][1] = 0;
  input[10][2] = 255;

  input[11][0] = 0;
  input[11][1] = 128;
  input[11][2] = 255;

  // Bright
  input[12][0] = 192;
  input[12][1] = 0;
  input[12][2] = 0;

  input[13][0] = 0;
  input[13][1] = 192;
  input[13][2] = 0;

  input[14][0] = 0;
  input[14][1] = 0;
  input[14][2] = 192;

  input[15][0] = 192;
  input[15][1] = 192;
  input[15][2] = 0;

  input[16][0] = 192;
  input[16][1] = 0;
  input[16][2] = 192;

  input[17][0] = 0;
  input[17][1] = 192;
  input[17][2] = 192;

  input[18][0] = 192;
  input[18][1] = 92;
  input[18][2] = 0;

  input[19][0] = 192;
  input[19][1] = 0;
  input[19][2] = 92;

  input[20][0] = 92;
  input[20][1] = 192;
  input[20][2] = 0;

  input[21][0] = 0;
  input[21][1] = 192;
  input[21][2] = 92;

  input[22][0] = 92;
  input[22][1] = 0;
  input[22][2] = 192;

  input[23][0] = 0;
  input[23][1] = 92;
  input[23][2] = 192;

  // Medium
  input[24][0] = 128;
  input[24][1] = 0;
  input[24][2] = 0;

  input[25][0] = 0;
  input[25][1] = 128;
  input[25][2] = 0;

  input[26][0] = 0;
  input[26][1] = 0;
  input[26][2] = 128;

  input[27][0] = 128;
  input[27][1] = 128;
  input[27][2] = 0;

  input[28][0] = 128;
  input[28][1] = 0;
  input[28][2] = 128;

  input[29][0] = 0;
  input[29][1] = 128;
  input[29][2] = 128;

  input[30][0] = 128;
  input[30][1] = 64;
  input[30][2] = 0;

  input[31][0] = 128;
  input[31][1] = 0;
  input[31][2] = 64;

  input[32][0] = 64;
  input[32][1] = 128;
  input[32][2] = 0;

  input[33][0] = 0;
  input[33][1] = 128;
  input[33][2] = 64;

  input[34][0] = 64;
  input[34][1] = 0;
  input[34][2] = 128;

  input[35][0] = 0;
  input[35][1] = 64;
  input[35][2] = 128;

  // Dark
  input[36][0] = 64;
  input[36][1] = 0;
  input[36][2] = 0;

  input[37][0] = 0;
  input[37][1] = 64;
  input[37][2] = 0;

  input[38][0] = 0;
  input[38][1] = 0;
  input[38][2] = 64;

  input[39][0] = 64;
  input[39][1] = 64;
  input[39][2] = 0;

  input[40][0] = 64;
  input[40][1] = 0;
  input[40][2] = 64;

  input[41][0] = 0;
  input[41][1] = 64;
  input[41][2] = 64;

  input[42][0] = 64;
  input[42][1] = 32;
  input[42][2] = 0;

  input[43][0] = 64;
  input[43][1] = 0;
  input[43][2] = 32;

  input[44][0] = 32;
  input[44][1] = 64;
  input[44][2] = 0;

  input[45][0] = 0;
  input[45][1] = 64;
  input[45][2] = 32;

  input[46][0] = 32;
  input[46][1] = 0;
  input[46][2] = 64;

  input[47][0] = 0;
  input[47][1] = 32;
  input[47][2] = 64;

  // set up window
  glutInitWindowSize(400, 400);
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutCreateWindow("Self-organizing Map Demo");

  // register callback functions
  glutDisplayFunc(display);
  glutReshapeFunc(reshape); 
  glutKeyboardFunc(keyboard);
  glutIdleFunc(idle);

  // initalize opengl parameters
  init();

  // loop until something happens
  glutMainLoop();

  // clean up garbage
  delete som;

  return 0;           
}

void idle(void)
{
	if (training && som->getEpoch() < RAND_MAX) {
		int randInput = rand() % numInputs;
		// Make it so that the bright colors are twice as likely
		if (randInput > 11)
			randInput = rand() % numInputs;
		som->train(input[randInput][0], input[randInput][1], input[randInput][2]);
		glutPostRedisplay();
	}
}

void init()
{
  // initialize viewing system
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45.0, 1.0, 1.0, 1000.0);
  glMatrixMode(GL_MODELVIEW);
 
  // initialize background color to black
  glClearColor(0.0,0.0,0.0,0.0);

  // enable depth buffering
  glEnable(GL_DEPTH_TEST);
}

void reshape(int width, int height)
{
  glViewport(0, 0, (GLsizei) width, (GLsizei) height);
}

void display()
{
	// clear buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// compute eye position
	glLoadIdentity();

	gluLookAt(50,50,200, 50,50,0, 0,1,0);

	// draw the current map
	som->display();

	// draw to screen
	glutSwapBuffers();
}

void keyboard(unsigned char key, int mouseX, int mouseY)
{ 
	switch (key)
	{
	case 't':
		training = !training;
		if (training)
			cout << "Training..." << endl;
		else
			cout << "Stopped training" << endl;
		break;
	case 'r':
		training = false;
		som = new Som(size, numInputs);
		cout << "Map reset" << endl;
		glutPostRedisplay();
		break;
	}
}