#include ".\som.h"
#include <math.h>
#include <GL/glut.h>
#include <iostream>
using namespace std;

double getDistance(int x1, int y1, int x2, int y2)
{
	return sqrt(pow(x1 - x2, 2) + pow(y1 - y2, 2));
}

Som::Som(int size, int numInputs) :
	size_(size),
	numInputs_(numInputs),
	iteration_(0),
	epoch_(0)
{
	map.resize(size);
	for (int i = 0; i < size; ++i)
		map[i].resize(size);

	// Randomize the map
	for (int i = 0; i < size; ++i) {
		for (int j = 0; j < size; ++j) {
			double randR = (double)rand() / (double)RAND_MAX;
			double randG = (double)rand() / (double)RAND_MAX;
			double randB = (double)rand() / (double)RAND_MAX;
			map[i][j].setR(randR);
			map[i][j].setG(randG);
			map[i][j].setB(randB);
			map[i][j].setX(i);
			map[i][j].setY(j);
		}
	}
}

Som::~Som()
{

}

void Som::train(double r, double g, double b)
{
	if (epoch_ >= RAND_MAX)
		return;

	double winneri, winnerj, distance;
	double closest = sqrt((255.0 * 255.0) + (255.0 * 255.0) + (255.0 * 255.0));  // Maximum deviation

	// iterate through the map
	for (int i = 0; i < size_; ++i) {
		for (int j = 0; j < size_; ++j) {
			// calculate the euclidean distance of the current neuron from the input
			distance = sqrt((pow(map[i][j].getR() - r, 2.0)) + 
							(pow(map[i][j].getG() - g, 2.0)) + 
							(pow(map[i][j].getB() - b, 2.0)));
			// if the distance is the shortest found so far, set winneri, winnerj, and closest
			if (distance < closest) {
				winneri = i;
				winnerj = j;
				closest = distance;
			}
			// if the distance is equal to the shortest found so far, randomly select whether to update
			else if (distance == closest) {
				double random = (double)rand() / (double)RAND_MAX;
				if (random < 0.5) {
					winneri = i;
					winnerj = j;
				}
			}
		}
	}

	// adjust nearby weights using a gaussian function
	for (int i = 0; i < size_; ++i) {
		for (int j = 0; j < size_; ++j) {
			// the adjustment i'm multiplying it by is to make its effect less drastic as time goes on
			double adjustment = (double)epoch_ / 100.0;
			if (adjustment >= 1)
				adjustment = 0.99;

			// the adjustment i'm dividing the distance by is to make its effect spread to less neighbors as time goes on
			// starts out at 10 (large radius) and decreases as time goes on
			double adjustment2 = 10.0 - (double)epoch_ / 10.0;
			if (adjustment2 <= 0)
				adjustment2 = 0.1;

			double gaussian = (1.0 - adjustment) * exp(-getDistance(winneri, winnerj, i, j) / adjustment2);
			double diffR = map[winneri][winnerj].getR() - map[i][j].getR();
			double diffG = map[winneri][winnerj].getG() - map[i][j].getG();
			double diffB = map[winneri][winnerj].getB() - map[i][j].getB();

			map[i][j].setR(map[i][j].getR() + (gaussian * diffR));
			map[i][j].setG(map[i][j].getG() + (gaussian * diffG));
			map[i][j].setB(map[i][j].getB() + (gaussian * diffB));
		}
	}

	// increase the counter.  if max number of inputs has been reached, increment the epoch number.
	++iteration_;
	if (iteration_ == numInputs_) {
		iteration_ = 0;
		++epoch_;
		cout << "Epoch " << epoch_ << endl;
	}
}

// Draw the SOM
void Som::display()
{
	for (int i = 0; i < size_; ++i) {
		for (int j = 0; j < size_; ++j) {
			glColor3f(map[i][j].getR(), map[i][j].getG(), map[i][j].getB());
			glBegin(GL_QUADS);
				glVertex3f(i, j, 0);			// upper left
				glVertex3f(i, j-1, 0);			// lower left
				glVertex3f(i+1, j-1, 0);		// lower right
				glVertex3f(i+1, j, 0);			// upper right
			glEnd();
		}
	}
}

void Som::reset()
{
	// Randomize the map
	for (int i = 0; i < size_; ++i) {
		for (int j = 0; j < size_; ++j) {
			double randR = (double)rand() / (double)RAND_MAX;
			double randG = (double)rand() / (double)RAND_MAX;
			double randB = (double)rand() / (double)RAND_MAX;
			map[i][j].setR(randR);
			map[i][j].setG(randG);
			map[i][j].setB(randB);
		}
	}
	iteration_ = 0;
	epoch_ = 0;
}

int Som::getEpoch()
{
	return epoch_;
}
